package moteView;

import java.awt.Graphics;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

public class DisplayPanel extends JPanel{
	private MoteCircle staticPoints[];
//	private MoteCircle variablePoints[];
	private BufferedImage icon;
	private BufferedImage locateIcon;
	private static final long serialVersionUID = 7526471155622776147L;
	private boolean DEBUG;
	
	private int margin = 50;
	
	private int factor;
	
	public DisplayPanel(MoteCircle[] sPoints, boolean debug) {
		super();
		
		DEBUG = debug;
		if(DEBUG) System.out.println("DisplayPanel():ENTER: ");
		
		staticPoints = sPoints;
		//refactorPoints(5);
		
		try {
			icon = javax.imageio.ImageIO.read(new java.io.File("C:\\Documents and Settings\\Tim\\workspace\\Mote Analyzer\\ball.blue.gif"));
		}
		catch(java.io.IOException ex){
			if(DEBUG) System.out.println("DisplayPanel():ERROR: No image found");
			JOptionPane.showMessageDialog(null,"No Image","No Image",JOptionPane.ERROR_MESSAGE);
		}
		
		try {
			locateIcon = javax.imageio.ImageIO.read(new java.io.File("C:\\Documents and Settings\\Tim\\workspace\\Mote Analyzer\\ball.red.gif"));
		}
		catch(java.io.IOException ex){
			if(DEBUG) System.out.println("DisplayPanel():ERROR: No image found");
			JOptionPane.showMessageDialog(null,"No Image","No Image",JOptionPane.ERROR_MESSAGE);
		}
		
		this.setPreferredSize(new Dimension(200,200));
		
		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				refactorPoints(findRefactor());
			}
		});
		
		if(DEBUG) System.out.println("DisplayPanel():EXIT: ");
	}
	
	public void updateRadius(double[] radiusArray) {
		if(DEBUG) System.out.println("DisplayPanel:updateRadius():ENTER: ");
		for(int i=0; i<staticPoints.length; i++) {
			staticPoints[i].setRadius(radiusArray[i]+factor);
		}
		this.validate();
		if(DEBUG) System.out.println("DisplayPanel:updateRadius():EXIT: ");
	}
	
	private void refactorPoints(int factor) {
		if(DEBUG) System.out.println("DisplayPanel:refactorPoints():ENTER: factor: "+factor);
		for(int i=0; i< staticPoints.length; i++) {
			MoteCircle oldPoint = staticPoints[i];
			MotePoint tempPoint = new MotePoint(staticPoints[i].getName(),staticPoints[i].getOrigin().getX()*factor,staticPoints[i].getOrigin().getY()*factor);
			staticPoints[i] = new MoteCircle(staticPoints[i].getName(),tempPoint, staticPoints[i].getRadius(),DEBUG);
			if(DEBUG) System.out.println("DisplayPanel:refactorPoints():DEBUG: refactored: "+oldPoint+" to: "+staticPoints[i]);
		
		}
		if(DEBUG) System.out.println("DisplayPanel:refactorPoints():EXIT:");
	}
	
	private int findRefactor() {
		if(DEBUG) System.out.println("DisplayPanel:findRefactor():ENTER: ");
		double leftMax=0, rightMax=0, bottomMax=0;
		for(int i=0; i< staticPoints.length; i++) {
			if(staticPoints[i].getOrigin().getX() < leftMax)
				leftMax = staticPoints[i].getOrigin().getX();
			if(staticPoints[i].getOrigin().getX() > rightMax)
				rightMax = staticPoints[i].getOrigin().getX();
			if(staticPoints[i].getOrigin().getY() > bottomMax)
				bottomMax = staticPoints[i].getOrigin().getY();
		}
		
		if(leftMax < 0) {
			if(DEBUG) System.out.println("DisplayPanel:findRefactor():DEBUG: shifting right by: "+leftMax);
			double shiftVal = Math.abs(leftMax);
			for(int i=0; i<4; i++) {
				MotePoint oldPt = staticPoints[i].getOrigin();
				MotePoint tempPt = new MotePoint(oldPt.getName(), oldPt.getX()+shiftVal, oldPt.getY());
				staticPoints[i] = new MoteCircle(staticPoints[i].getName(), tempPt, staticPoints[i].getRadius(),DEBUG);
			}
		}
		
		int width =(int) rightMax - (int) leftMax;
		int height =(int) bottomMax;
		
		int windowWidth = this.getWidth()-margin*2;
		int windowHeight = this.getHeight()-margin*2;
		
		int widthRatio = windowWidth / width;
		int heightRatio = windowHeight / height;
		
		if(widthRatio < heightRatio) {
			if(DEBUG) System.out.println("DisplayPanel:findRefactor():EXIT: ratio: "+widthRatio);
			factor = widthRatio;
			return(widthRatio);
		}
		else {
			if(DEBUG) System.out.println("DisplayPanel:findRefactor():EXIT: ratio: "+heightRatio);
			factor = heightRatio;
			return(heightRatio);
		}
		
	}
	
	//public void setVariblePoints(MoteCircle[] points) {
	//	variablePoints = points;
	//}
	
	
	public void paintComponent(Graphics g){
		super.paintComponent(g);
		if(DEBUG) System.out.println("DisplayPanel:paintComponent():ENTER: ");
		
	//	int imgWidth = image.getWidth(this);
	//	int imgHeight = image.getWidth(this);
		
		//icon.paintIcon((Component)icon.getImageObserver(), g, 5, 5);
		//icon.paintIcon(null, g, 20, 20);
		
		
		//draw static points with icons
		for(int i=0; i< staticPoints.length; i++) {
			//g.drawImage(icon,(int)staticPoints[i].getOrigin().getX(),(int)staticPoints[i].getOrigin().getY(),null);
			drawImage(icon,staticPoints[i].getOrigin(),g);
					
			if(staticPoints[i].getRadius() > 0)
				drawCircle(staticPoints[i],g);
		}
		
		//draw circles
		//for(int i=0; i<variablePoints.length; i++) {
			
		//}
		if(DEBUG) System.out.println("DisplayPanel:paintComponent():EXIT: ");
	}
	
	private void drawCircle(MoteCircle circle, Graphics g) {
		if(DEBUG) System.out.println("DisplayPanel:drawCircle():ENTER: circle: "+circle);
		double radius = circle.getRadius();
		//double deltaX = radius * Math.sin(Math.PI/4d);
		//double deltaY = radius * Math.cos(Math.PI/4d);
		double deltaX = radius/2;
		double deltaY = radius/2;
		
		g.drawOval((int)(circle.getOrigin().getX()-deltaX+margin), (int)(circle.getOrigin().getY()-deltaY+margin), (int)radius, (int)radius);
		if(DEBUG) System.out.println("DisplayPanel:drawCircle():EXIT: circle: "+circle);
	}
	
	private void drawImage(BufferedImage img, MotePoint loc, Graphics g) {
		double width = img.getWidth()/2;
		double height = img.getHeight()/2;
		//double deltaX = radius * Math.sin(Math.PI/4d);
		//double deltaY = radius * Math.cos(Math.PI/4d);
		//double deltaX = radius/2;
		//double deltaY = radius/2;
		
		g.drawImage(img,(int)(loc.getX()-width+margin), (int)(loc.getY()-height+margin),null);
	}
}
