package moteView;

public class MoteCircle {
	private MotePoint origin;
	private double radius;
	private String name;
	private boolean DEBUG;
	
	public MoteCircle(String name, MotePoint o, double r, boolean debug) {
		DEBUG = debug;
		origin = o;
		radius = r;
		this.name = name;
	}
	
	public String getName() {
		return(name);
	}
	
	public double getRadius() {
		return(radius);
	}
	
	public void setRadius(double radius) {
		this.radius = radius;
	}
	
	public MotePoint getOrigin() {
		return(origin);
	}
	
	public String toString() {
		return("Circle: "+name+" origin: "+origin+" radius: "+radius);
	}
	
	public boolean containsPoint(MotePoint x) {
		if(DEBUG) System.out.println("MoteCircle:containsPoint():ENTER: this: "+this+" x: "+x);
		double distanceAB = MoteViewController.getDistance(this.origin, x);
		if(distanceAB < radius) {
			if(DEBUG) System.out.println("MoteCircle:containsPoint():EXIT: "+true);
			return(true);
		}
		else {
			if(DEBUG) System.out.println("MoteCircle:containsPoint():EXIT: "+false);
			return(false);
		}
	}
	
	public boolean containsEntireCircle(MoteCircle x) {
		if(DEBUG) System.out.println("MoteCircle:containsEntireCirce():ENTER: this: "+this+" x: "+x);
		double distanceAB = MoteViewController.getDistance(this.origin, x.getOrigin());
		if((distanceAB + x.getRadius()) < radius) {
			if(DEBUG) System.out.println("MoteCircle:containsEntireCircle():EXIT: "+true);
			return(true);
		}
		else
			if(DEBUG) System.out.println("MoteCircle:containsEntireCircle():EXIT: "+false);
			return(false);
	}
	
	public boolean noIntersection(MoteCircle x) {
		if(DEBUG) System.out.println("MoteCircle:noIntersection():ENTER: this: "+this+" x: "+x);
		double distanceAB = MoteViewController.getDistance(this.origin, x.getOrigin());
		if(containsEntireCircle(x)) {
			if(DEBUG) System.out.println("MoteCircle:noIntersection():EXIT: "+true);
			return(true);
		}
		else if(!containsPoint(x.getOrigin())) {
			if(distanceAB > (radius + x.getRadius())) {
				if(DEBUG) System.out.println("MoteCircle:noIntersection():EXIT: "+true);
				return(true);
			}
		}
		if(DEBUG) System.out.println("MoteCircle:noIntersection():EXIT: "+false);
		return(false);
	}
	
	public boolean oneIntersection(MoteCircle x) {
		if(DEBUG) System.out.println("MoteCircle:oneIntersection():ENTER: this: "+this+" x: "+x);
		double distanceAB = MoteViewController.getDistance(this.origin, x.getOrigin());
		if(containsPoint(x.getOrigin())) {
			if(radius == (distanceAB + x.getRadius())) {
				if(DEBUG) System.out.println("MoteCircle:oneIntersection():EXIT: "+true);
				return(true);
			}
		}
		else {
			if(distanceAB == (radius + x.getRadius())) {
				if(DEBUG) System.out.println("MoteCircle:oneIntersection():EXIT: "+true);
				return(true);
			}
		}
		if(DEBUG) System.out.println("MoteCircle:oneIntersection():EXIT: "+false);
		return(false);
	}
	
	public MotePoint[] calculateIntersectionPoints(MoteCircle b) {
		if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():ENTER: this: "+this+" b: "+b);
		//if b > a, swap names
		if(b.getRadius() > radius) {
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():DEBUG: b is bigger than a, swap");
			MotePoint[] retVals = b.calculateIntersectionPoints(this);
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():EXIT: number of points: "+(retVals==null?0:retVals.length));
			return(retVals);
		}
		
		if(noIntersection(b)) {
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():DEBUG: no intersection");
			double distanceAB = MoteViewController.getDistance(this.origin, b.getOrigin());
			MotePoint temp1 = new MotePoint("Intersect Point 1: A: "+this+" B: "+b,radius,0d);
			MotePoint temp2 = new MotePoint("Intersect Point 2: A: "+this+" B: "+b,distanceAB-b.radius,0d);
			
			//get angle to rotate by
			double angle = MoteViewController.getAngleFromAToB(this.origin, b.origin);
			
			//rotate
			temp1 = MoteViewController.rotatePoint(temp1, angle);
			temp2 = MoteViewController.rotatePoint(temp2, angle);
			
			//translate
			
			temp1 = MoteViewController.translatePoint(temp1, origin.getX(), origin.getY());
			temp2 = MoteViewController.translatePoint(temp2, origin.getX(), origin.getY());
			
			MotePoint[] returnVals = new MotePoint[2];
			returnVals[0] = temp1;
			returnVals[1] = temp2;
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():EXIT: returning 2 points");
			return(returnVals);
		}
		if(oneIntersection(b)) {
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():DEBUG: one intersection");
			MotePoint temp1 = new MotePoint("Intersect Point 1: A: "+this+" B: "+b,radius,0d);
			
//			get angle to rotate by
			double angle = MoteViewController.getAngleFromAToB(this.origin, b.origin);
			
			//rotate
			temp1 = MoteViewController.rotatePoint(temp1, angle);
			
			//translate
			
			temp1 = MoteViewController.translatePoint(temp1, origin.getX(), origin.getY());
			
			MotePoint[] returnVals = new MotePoint[1];
			returnVals[0] = temp1;
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():EXIT: returning 1 point");
			return(returnVals);
		}
		else {
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():DEBUG: two intersection");
			double distanceAB = MoteViewController.getDistance(this.origin, b.getOrigin());
			
			//find angle opposite AB
			// c^2 = a^2 +b^2 - 2ab cos theta ; where theta is opposite side c
			double top = (-Math.pow(distanceAB, 2) + Math.pow(this.radius, 2) + Math.pow(b.radius, 2));
			double bottom = (2 * this.radius + b.radius);
			double theta = Math.acos((top/bottom));
			
			double deltaX = this.radius * Math.asin(theta);
			double deltaY = this.radius * Math.acos(theta);
			
			MotePoint temp1 = new MotePoint("Intersect Point 1: A: "+this+" B: "+b,deltaX,deltaY);
			MotePoint temp2 = new MotePoint("Intersect Point 2: A: "+this+" B: "+b,deltaX,-deltaY);
			
//			get angle to rotate by
			double angle = MoteViewController.getAngleFromAToB(this.origin, b.origin);
			
			//rotate
			temp1 = MoteViewController.rotatePoint(temp1, angle);
			temp2 = MoteViewController.rotatePoint(temp2, angle);
			
			//translate
			
			temp1 = MoteViewController.translatePoint(temp1, origin.getX(), origin.getY());
			temp2 = MoteViewController.translatePoint(temp2, origin.getX(), origin.getY());
			
			MotePoint[] returnVals = new MotePoint[2];
			returnVals[0] = temp1;
			returnVals[1] = temp2;
			if(DEBUG) System.out.println("MoteCircle:calculateIntersectionPoints():EXIT: returning 2 points");
			return(returnVals);
		}
	}
	
}
