package moteView;

import java.awt.*;

public class MoteLine {
	
	//y = slope * x + y_int
	
	Point x,y;
	
	double slope;
	
	double y_int;
	
	double angleInRadians;
	
	private boolean isVertical;
	
	private double xValue;
	
	private double tolerence = .25;
	
	private boolean DEBUG;
	
	public MoteLine(Point x, Point y, boolean debug) {
		DEBUG = debug;
		if(DEBUG) System.out.println("MoteLine():ENTER: x: "+x+" y: "+y);
		if(x == null) {
			if(DEBUG) System.out.println("MoteLine():ERROR: x is null");
			throw new AssertionError("MoteLine(): Point x cannot be null");
		}
		if(y == null) {
			if(DEBUG) System.out.println("MoteLine():ERROR: y is null");
			throw new AssertionError("MoteLine(): Point y cannot be null");
		}
		if(x.equals(y)) {
			if(DEBUG) System.out.println("MoteLine():ERROR: x and y are equal");
			throw new AssertionError("MoteLine(): Point x and y must be distinct");
		}
		this.x = x;
		this.y = y;
		
		//calc slope
		calculateSlope();
		
		//calc y_int
		calculateIntercept();
		
		//calc radians from X axis
		calculateAngleInRadians();
		if(DEBUG) System.out.println("MoteLine():EXIT: x: "+x+" y: "+y);
	}
	
	private void calculateSlope() {
		if(DEBUG) System.out.println("MoteLine:CalculateSlope():ENTER:");
		double x1 = x.getX();
		double x2 = y.getX();
		double y1 = x.getY();
		double y2 = y.getY();
		
		if(y2 == y1) {
			isVertical = true;
			xValue = x1;
			if(DEBUG) System.out.println("MoteLine:CalculateSlope():DEBUG: vertical line on x at: "+x1);
		}
		else {
			
			isVertical = false;
			slope = (x2-x1) / (y2-y1);
			if(DEBUG) System.out.println("MoteLine:CalculateSlope():DEBUG: slope is: "+slope);
		}
		if(DEBUG) System.out.println("MoteLine:CalculateSlope():ENTER:");
	}
	
	private void calculateIntercept() {
		if(DEBUG) System.out.println("MoteLine:CalculateIntercept():ENTER: ");
		if(isVertical) {
			if(DEBUG) System.out.println("MoteLine:CalculateIntercept():EXIT: vertical line, no y-intercept");
			return;
		}
		double y1= x.getY();
		double x1= x.getX();
		y_int = y1 - (slope * x1);
		if(DEBUG) System.out.println("MoteLine:CalculateIntercept():DEBUG: y-intercept is: "+y_int);
		if(DEBUG) System.out.println("MoteLine:CalculateIntercept():EXIT: ");
	}
	
	private void calculateAngleInRadians() {
		if(DEBUG) System.out.println("MoteLine:CalculateAngleInRadians():ENTER: ");
		double deltaX = y.getX() - x.getX();
		double deltaY = y.getY() - x.getY();
		angleInRadians = Math.tan(deltaX/deltaY);
		if(DEBUG) System.out.println("MoteLine:CalculateAngleInRadians():DEBUG: angleInRadians is: "+angleInRadians);
		if(DEBUG) System.out.println("MoteLine:CalculateAngleInRadians():EXIT: ");
	}
	
	public Point[] getPointsXDistanceFromY(double xDistance, Point yPoint) throws PointLocationException{
		if(DEBUG) System.out.println("MoteLine:GetPointXDistanceFromY():ENTER: xDistance: "+xDistance+" yPoint: "+yPoint);
		
		if(!isPointOnLine(y))
			throw new PointLocationException("point: "+y+" is not on line: "+toString());
		
		Point[] returnPoints = new Point[2];
		if(isVertical) {
			returnPoints[0] = new Point(); 
			returnPoints[0].setLocation(yPoint.getX()+xDistance, yPoint.getY());
			returnPoints[1] = new Point(); 
			returnPoints[1].setLocation(yPoint.getX()-xDistance, yPoint.getY());
		}
		else {
			double deltaX = xDistance * Math.asin(angleInRadians);
			double deltaY = xDistance * Math.acos(angleInRadians);
			returnPoints[0] = new Point();
			returnPoints[0].setLocation(yPoint.getX()+deltaX, yPoint.getY()+deltaY);
			returnPoints[1] = new Point();
			returnPoints[1].setLocation(yPoint.getX()-deltaX, yPoint.getY()-deltaY);
		}
		if(DEBUG) System.out.println("MoteLine:GetPointXDistanceFromY():DEBUG: point0: "+returnPoints[0]+" point1: "+returnPoints[1]);
		if(DEBUG) System.out.println("MoteLine:GetPointXDistanceFromY():EXIT: xDistance: "+xDistance+" yPoint: "+yPoint);
		return(returnPoints);
	}
	
	public boolean isPointOnLine(Point pt) {
		if(DEBUG) System.out.println("MoteLine:isPointOnLine():ENTER: pt: "+pt+" line: "+this);
		if(isVertical) {
			if(DEBUG) System.out.println("MoteLine:isPointOnLine():EXIT: "+(x.getX() == xValue));
			return(x.getX() == xValue);
		}
		else
		{
			double yValue = slope * pt.getX() + y_int;
			if((((pt.getY()-tolerence) <= yValue) || (yValue <= (pt.getY()+tolerence)))){
				if(DEBUG) System.out.println("MoteLine:isPointOnLine():EXIT: "+true);
				return(true);
			}
		}
		if(DEBUG) System.out.println("MoteLine:isPointOnLine():EXIT: "+false);
		return(false);
	}
	
	public String toString() {
		StringBuffer buff = new StringBuffer();
		buff.append("y = "+slope+" * x + "+y_int);
		return(buff.toString());
	}
}
