package moteView;

import java.io.IOException;
import java.net.InetAddress;

import javax.swing.JOptionPane;

//import moteAnalyzer.SocketListener;

public class MoteViewController extends Thread implements ByteStreamReciever{
	
	public static final int MAX_STATIC_MOTES = 4;

	private MoteViewGui gui;
	
	private MotePoint[] staticPointList;
	
	private MoteCircle[] stationaryMotes;
	
	private static boolean DEBUG;
	
	private SocketListener socketListener;
	
	private RatioTable ratioTable;
	
	private boolean displayCreated = false;
	
	private long refreshRate = 10000;
	
	private DisplayPanel displayPanel;
	
	private boolean doMoteDataUpdate = true;
	
	public MoteViewController(boolean debug) {
		DEBUG = debug;
		staticPointList = new MotePoint[MAX_STATIC_MOTES];
		ratioTable = new RatioTable(DEBUG);
		start();
	}
	
	public void setGui(MoteViewGui g) {
		gui = g;
	}
	
	public void loadConfiguration(String d01, String d02, String d12, String d03, String d13, String d23) {
		if(DEBUG) System.out.println("MoteViewController:loadConfiguration():ENTER: d01: "+
				d01+" d02: "+d02+" d12: "+d12+" d03: "+d03+" d13: "+d13+" d23: "+d23);
		
		double distance0To1;
		try {
			distance0To1 = Double.parseDouble(d01);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 0 to 1\": "
					+d01+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d01: "+
					d01+" is threw a NumberFormatParseException");
			return;
		}
		ratioTable.setPhysicalDistance(0, 1, distance0To1);
		
		double distance0To2;
		try {
			distance0To2 = Double.parseDouble(d02);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 0 to 2\": "
					+d02+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d02: "+
					d02+" is threw a NumberFormatParseException");
			return;
		}
		ratioTable.setPhysicalDistance(0, 2, distance0To2);
		
		double distance0To3;
		try {
			distance0To3 = Double.parseDouble(d03);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 0 to 3\": "
					+d03+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d03: "+
					d03+" is threw a NumberFormatParseException");
			return;
		}
		ratioTable.setPhysicalDistance(0, 3, distance0To3);
		
		double distance1To2;
		try {
			distance1To2 = Double.parseDouble(d12);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 1 to 2\": "
					+d12+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d12: "+
					d12+" is threw a NumberFormatParseException");
			return;
		}
		ratioTable.setPhysicalDistance(1, 2, distance1To2);
		
		double distance1To3;
		try {
			distance1To3 = Double.parseDouble(d13);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 1 to 3\": "
					+d13+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d13: "+
					d13+" is threw a NumberFormatParseException");
			return;
		}
		ratioTable.setPhysicalDistance(1, 3, distance1To3);
		
		double distance2To3;
		try {
			distance2To3 = Double.parseDouble(d23);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 2 to 3\": "
					+d23+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d23: "+
					d23+" is threw a NumberFormatParseException");
			return;
		}
		ratioTable.setPhysicalDistance(2, 3, distance2To3);
		if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT:");
	}
	
	public void findLocations(/*String d01, String d02, String d12, String d03, String d13, String d23*/) {
		//if(DEBUG) System.out.println("MoteViewController:loadConfiguration():ENTER: d01: "+
		//		d01+" d02: "+d02+" d12: "+d12+" d03: "+d03+" d13: "+d13+" d23: "+d23);
		if(DEBUG) System.out.println("MoteViewController:findLocations():");
		//first mote point, initially at 0,0 -- potentially translated later
		
		staticPointList[0] = new MotePoint("Mote 0",0d,0d);
		if(DEBUG) System.out.println("MoteViewController:findLocations():DEBUG: Static Point added: "+staticPointList[0]);
		
		//second mote point makes a line with the first mote point that is parallel to the top
		double distance0To1 = (double)ratioTable.getDistanceValue(0,1);
		staticPointList[1] = new MotePoint("Mote 1",distance0To1,0d);
		if(DEBUG) System.out.println("MoteViewController:findLocations():DEBUG: Static Point added: "+staticPointList[1]);
		/*try {
			distance0To1 = Double.parseDouble(d01);
			staticPointList[1] = new MotePoint("Mote 1",distance0To1,0d);
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():DEBUG: Static Point added: "+staticPointList[1]);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 0 to 1\": "
					+d01+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d01: "+
					d01+" is threw a NumberFormatParseException");
			return;
		}*/
		
		//use law of cosines to find deltaY1
		double distance0To2 = (double)ratioTable.getDistanceValue(0,2);
		/*try {
			distance0To2 = Double.parseDouble(d02);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 0 to 2\": "
					+d02+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d02: "+
					d02+" is threw a NumberFormatParseException");
			return;
		}*/
		double distance1To2 = (double)ratioTable.getDistanceValue(1,2);
		/*try {
			distance1To2 = Double.parseDouble(d12);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 1 to 2\": "
					+d12+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d12: "+
					d12+" is threw a NumberFormatParseException");
			return;
		}*/
		double top = (-Math.pow(distance1To2, 2) + Math.pow(distance0To1, 2) + Math.pow(distance0To2, 2));
		double bottom = 2d * distance0To1 * distance0To2;
		double angleBetween1And2 = Math.acos(top/bottom);
		if(angleBetween1And2 < (Math.PI/2d)) {
			angleBetween1And2 = (Math.PI/2d) - angleBetween1And2;
			double deltaY2 = distance0To2 * Math.acos(angleBetween1And2);
			double deltaX2 = distance0To2 * Math.asin(angleBetween1And2);
			staticPointList[2] = new MotePoint("Mote 2", deltaX2, deltaY2);
			if(DEBUG) System.out.println("MoteViewController:findLocations():DEBUG: Static Point added: "+staticPointList[2]);
		}
		else if(angleBetween1And2 > (Math.PI/2d)) {
			double deltaY2 = distance0To2 * Math.acos(angleBetween1And2);
			double deltaX2 = distance0To2 * Math.asin(angleBetween1And2);
			staticPointList[2] = new MotePoint("Mote 2", deltaX2, deltaY2);
			if(DEBUG) System.out.println("MoteViewController:findLocations():DEBUG: Static Point added: "+staticPointList[2]);
		}
		
//		use law of cosines to find deltaY1
		double distance0To3 = (double)ratioTable.getDistanceValue(0,3);
		/*try {
			distance0To3 = Double.parseDouble(d03);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 0 to 3\": "
					+d03+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d03: "+
					d03+" is threw a NumberFormatParseException");
			return;
		}*/
		double distance1To3 = (double)ratioTable.getDistanceValue(1,3);
		/*try {
			distance1To3 = Double.parseDouble(d13);
		}
		catch(NumberFormatException ex) {
			gui.displayInfoDialog("Invalid Number Format","\"Distance: 1 to 3\": "
					+d13+" is not a parseable double");
			if(DEBUG) System.out.println("MoteViewController:loadConfiguration():EXIT: d13: "+
					d13+" is threw a NumberFormatParseException");
			return;
		}*/
		top = (-Math.pow(distance1To3, 2) + Math.pow(distance0To1, 2) + Math.pow(distance0To3, 2));
		bottom = 2d * distance0To1 * distance0To3;
		double angleBetween0And3 = Math.acos(top/bottom);
		
		if(angleBetween0And3 < (Math.PI/2d)) {
			angleBetween0And3 = (Math.PI/2d) - angleBetween0And3;
			double deltaY3 = distance0To3 * Math.acos(angleBetween0And3);
			double deltaX3 = distance0To3 * Math.asin(angleBetween0And3);
			staticPointList[3] = new MotePoint("Mote 3", deltaX3, deltaY3);
			if(DEBUG) System.out.println("MoteViewController:findLocations():DEBUG: Static Point added: "+staticPointList[3]);
		}
		else if(angleBetween0And3 > (Math.PI/2d)) {
			double deltaY3 = distance0To3 * Math.acos(angleBetween0And3);
			double deltaX3 = distance0To3 * Math.asin(angleBetween0And3);
			staticPointList[3] = new MotePoint("Mote 3", deltaX3, deltaY3);
			if(DEBUG) System.out.println("MoteViewController:findLocations():DEBUG: Static Point added: "+staticPointList[3]);
		}
		
		/*double deltaY3 = distance0To3 * Math.asin(angleBetween0And3);
		double deltaX3 = (deltaY3 / (Math.tan(angleBetween0And3)));
		staticPointList[3] = new MotePoint("Mote 3", deltaX3, deltaY3);
		if(DEBUG) System.out.println("MoteViewController:findLocations():DEBUG: Static Point added: "+staticPointList[3]);
		*/
		
		stationaryMotes = new MoteCircle[4];
		for(int i=0; i<staticPointList.length; i++) {
			stationaryMotes[i] = new MoteCircle(staticPointList[i].getName(),staticPointList[i],0d,DEBUG); 
		}
		//DisplayPanel newPanel = new DisplayPanel(stationaryMotes,DEBUG);
		//gui.addDisplayTab(newPanel, "Point View");
		
		if(DEBUG) System.out.println("MoteViewController:findLocations():EXIT: ");
	}
	
	public void createDisplayPanel() {
		if(DEBUG) System.out.println("MoteViewController:createDisplayPanel():ENTER: ");
		findLocations();
		displayPanel = new DisplayPanel(stationaryMotes,DEBUG);
		gui.addDisplayTab(displayPanel, "View");
		displayCreated = true;
		if(DEBUG) System.out.println("MoteViewController:createDisplayPanel():EXIT: ");
	}
	
	public static double getDistance(MotePoint a, MotePoint b) {
		if(DEBUG) System.out.println("MoteViewController:getDistance():ENTER: a: "+a+" b: "+b);
		double retVal = a.distance(b);
		if(DEBUG) System.out.println("MoteViewController:getDistance():EXIT: distance: "+retVal);
		return(retVal);
	}
	
	public static MotePoint rotatePoint(MotePoint a, double angle) {
		if(DEBUG) System.out.println("MoteViewController:rotatePoint():ENTER: a: "+a+" angle: "+angle);
		double x = a.getX();
		double y = a.getY();
		
		double xPrime = (x * Math.cos(angle)) - (y * Math.sin(angle));
		double yPrime = (x * Math.sin(angle)) + (y * Math.cos(angle));
		
		MotePoint newPoint = new MotePoint(a.getName(),xPrime,yPrime);
		if(DEBUG) System.out.println("MoteViewController:rotatePoint():EXIT: returning: "+newPoint);
		return(newPoint);
	}
	
	public static MotePoint translatePoint(MotePoint a, double translateX, double translateY) {
		if(DEBUG) System.out.println("MoteViewController:translatePoint():ENTER: a: "+a+" translateX: "+translateX+" translateY: "+translateY);
		MotePoint newPoint = new MotePoint(a.getName(), a.getX()+translateX, a.getY()+translateY);
		if(DEBUG) System.out.println("MoteViewController:translatePoint():EXIT: returning: "+newPoint);
		return(newPoint);
	}
	
	public static double getAngleFromAToB(MotePoint a, MotePoint b) {
		if(DEBUG) System.out.println("MoteViewController:getAngleFromAToB():ENTER: a: "+a+" b: "+b);
		
		double deltaX = a.getX() - b.getX();
		double deltaY = a.getY() - b.getY();
		
		double angle = Math.atan((deltaX/deltaY));
		if(DEBUG) System.out.println("MoteViewController:getAngleFromAToB():EXIT: angle: "+angle);
		return(angle);
		
	}
	
	public void recieveByteStream(byte[] bStream) {
		if(DEBUG) System.out.println("MoteViewController:recieveByteStream():ENTER:");
		StringBuffer buf = new StringBuffer();
		try {
			if(bStream[8] != 0x1C) {
				if(DEBUG) System.out.println("MoteViewController:recieveByteStream():EXIT: tossing packet");
				return;
			}
		}
		catch(Exception ex) {
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():EXIT: tossing packet");
			return;
		}
		int[] byteArray = new int[21-9];
		/*for(int i=0; i<10; i++) {
			byte b = bStream[i];
			buf.append(Integer.toHexString(b));
			buf.append(":");
		}*/
		for(int i=9; i< 21; i++) {
			int b = bStream[i];
			//b -= 0xA0;
			//b = (byte)128;
			if(b<0) {
				b = 128+(b + 128);
				
			}
			byteArray[i-9] = b;
			buf.append(Integer.toHexString(b));
			buf.append(":");
		}
		/*for(int i=0; i<bStream.length; i++) {
			byte b = bStream[i];
			buf.append(b+"="+Integer.toHexString(b));
			buf.append(":");
		}*/
		if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: bStream: "+buf);
		
		// take info and update structures
		
		
		switch(byteArray[0]) {
		case 0:
			int signal01 = byteArray[4] << 8;
			signal01 = signal01 | byteArray[5];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 0 to 1: "+signal01);
			ratioTable.setSignalStrength(0, 1, signal01);
			int signal02 = byteArray[6] << 8;
			signal02 = signal02 | byteArray[7];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 0 to 2: "+signal02);
			ratioTable.setSignalStrength(0, 2, signal02);
			int signal03 = byteArray[8] << 8;
			signal03 = signal03 | byteArray[9];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 0 to 3: "+signal03);
			ratioTable.setSignalStrength(0, 3, signal03);
			int signal04 = byteArray[10] << 8;
			signal04 = signal04 | byteArray[11];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 0 to 4: "+signal04);
			ratioTable.setSignalStrength(0, 4, signal04);
			break;
		case 1:
			int signal10 = byteArray[2] << 8;
			signal10 = signal10 | byteArray[3];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 1 to 0: "+signal10);
			ratioTable.setSignalStrength(1, 0, signal10);
			int signal12 = byteArray[6] << 8;
			signal12 = signal12 | byteArray[7];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 1 to 2: "+signal12);
			ratioTable.setSignalStrength(1, 2, signal12);
			int signal13 = byteArray[8] << 8;
			signal13 = signal13 | byteArray[9];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 1 to 3: "+signal13);
			ratioTable.setSignalStrength(1, 3, signal13);
			int signal14 = byteArray[10] << 8;
			signal14 = signal14 | byteArray[11];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 1 to 4: "+signal14);
			ratioTable.setSignalStrength(1, 4, signal14);
			break;
		case 2:
			int signal20 = byteArray[2] << 8;
			signal20 = signal20 | byteArray[3];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 2 to 0: "+signal20);
			ratioTable.setSignalStrength(2, 0, signal20);
			int signal21 = byteArray[4] << 8;
			signal21 = signal21 | byteArray[5];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 2 to 1: "+signal21);
			ratioTable.setSignalStrength(2, 1, signal21);
			int signal23 = byteArray[8] << 8;
			signal23 = signal23 | byteArray[9];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 2 to 3: "+signal23);
			ratioTable.setSignalStrength(2, 3, signal23);
			int signal24 = byteArray[10] << 8;
			signal24 = signal24 | byteArray[11];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 2 to 4: "+signal24);
			ratioTable.setSignalStrength(2, 4, signal24);
			break;
		case 3:
			int signal30 = byteArray[2] << 8;
			signal30 = signal30 | byteArray[3];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 3 to 0: "+signal30);
			ratioTable.setSignalStrength(3, 0, signal30);
			int signal31 = byteArray[4] << 8;
			signal31 = signal31 | byteArray[5];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 3 to 1: "+signal31);
			ratioTable.setSignalStrength(3, 1, signal31);
			int signal32 = byteArray[6] << 8;
			signal32 = signal32 | byteArray[7];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 3 to 2: "+signal32);
			ratioTable.setSignalStrength(3, 2, signal32);
			int signal34 = byteArray[10] << 8;
			signal34 = signal34 | byteArray[11];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 3 to 4: "+signal34);
			ratioTable.setSignalStrength(3, 4, signal34);
			break;
		case 4:
			int signal40 = byteArray[2] << 8;
			signal40 = signal40 | byteArray[3];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 4 to 0: "+signal40);
			ratioTable.setSignalStrength(4, 0, signal40);
			int signal41 = byteArray[4] << 8;
			signal41 = signal41 | byteArray[5];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 4 to 1: "+signal41);
			ratioTable.setSignalStrength(4, 1, signal41);
			int signal42 = byteArray[6] << 8;
			signal42 = signal42 | byteArray[7];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 4 to 2: "+signal42);
			ratioTable.setSignalStrength(4, 2, signal42);
			int signal43 = byteArray[8] << 8;
			signal43 = signal43 | byteArray[9];
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: signal 4 to 3: "+signal43);
			ratioTable.setSignalStrength(4, 3, signal43);
			break;
		}
		//gui.appendTextMessage(buf.toString());
		if(ratioTable.isReady() && !displayCreated) {
		//if(DEBUG && !displayCreated) {
			if(DEBUG) System.out.println("MoteViewController:recieveByteStream():DEBUG: ratioTable is ready");
			ratioTable.calculateRatio();
			createDisplayPanel();
		}
		if(DEBUG) System.out.println("MoteViewController:recieveByteStream():EXIT:");
	}
	
	public void handleIOException(IOException ex) {
		if(DEBUG) System.out.println("MoteViewController:handleIOException():ENTER: bStream:");
		if(DEBUG) System.out.println("MoteViewController:handleIOException():ERROR: IOException: "+ex.getMessage());
		JOptionPane.showMessageDialog(gui, "Exception Reading Socket - \n"+ex.getMessage()+ 
				"\n"+ex.getStackTrace(),"IO Exception",JOptionPane.ERROR_MESSAGE);
		if(DEBUG) System.out.println("MoteViewController:handleIOException():EXIT: bStream:");
	}
	
	public void startListening(String addr, String port) {
		if(DEBUG) System.out.println("MoteViewController:startListening():ENTER: address: "+addr+" port: "+port);
		if(socketListener == null) {
			socketListener = new SocketListener(DEBUG);
		}
		else {
			socketListener.closeSocket();
		}
		try {
			socketListener.connectSocket(InetAddress.getByName(addr),Integer.parseInt(port));
			socketListener.registerByteStreamReciever(this);
			socketListener.start();
			
		}
		catch(Exception ex) {
			if(DEBUG) System.out.println("MoteViewController:startListening():ERROR: IOException: "+ex.getMessage());
			JOptionPane.showMessageDialog(gui, "Unable to start connection - \n"+ex.getMessage()+ 
					"\n"+ex.getStackTrace(),"Connection Error",JOptionPane.ERROR_MESSAGE);
			
		}
		if(DEBUG) System.out.println("MoteViewController:startListening():EXIT: address: "+addr+" port: "+port);
	}
	
	public void run() {
		if(DEBUG) System.out.println("MoteViewController:run():ENTER:");
		long waitTime = System.currentTimeMillis() + refreshRate;
		do {
			if(System.currentTimeMillis() >= waitTime) {
				if(DEBUG) System.out.println("MoteViewController:run():DEBUG: update time");
				waitTime = System.currentTimeMillis()+refreshRate;
				if(displayCreated) 
					updateMoteData();
			}
		}while(doMoteDataUpdate);
		if(DEBUG) System.out.println("MoteViewController:run():EXIT:");
	}
	
	public void updateMoteData() {
		if(DEBUG) System.out.println("MoteViewController:updateMoteData():ENTER:");
		double[] radiusArray = new double[4];
		for(int i=0; i<4; i++) {
			radiusArray[i] = ratioTable.getMoteRadius(i);
		}
		displayPanel.updateRadius(radiusArray);
		if(DEBUG) System.out.println("MoteViewController:updateMoteData():EXIT: "+radiusArray[0]+","+radiusArray[1]+","+radiusArray[2]+","+radiusArray[3]);
	}
}
